@@ -1,14 +1,16 @@
 import datetime
-from crypt import crypt
 
-from sqlalchemy import Column, String, Text, DateTime, Integer
+from sqlalchemy import Column, Integer, String, DateTime, Text, ForeignKey
+from sqlalchemy.orm import relationship, backref
+from sqlalchemy.ext.hybrid import hybrid_property
 
-from uffd.ldapalchemy.dbutils import DBRelationship
 from uffd.database import db
-from uffd.ldap import ldap
+from uffd.tasks import cleanup_task
 from uffd.user.models import User
 from uffd.utils import token_urlfriendly
+from uffd.password_hash import PasswordHashAttribute, LowEntropyPasswordHash
 
+@cleanup_task.delete_by_attribute('expired_and_not_completed')
 class Signup(db.Model):
 	'''Model that represents a self-signup request
 
@@ -23,8 +25,8 @@ class Signup(db.Model):
 	address does not allow a third party to complete the signup procedure and
 	set a new password with the (also mail-based) password reset functionality.
 
-	As long as they are not completed, signup requests have no effect each other
-	or different parts of the application.'''
+	As long as they are not completed, signup requests have no effect on each
+	other or different parts of the application.'''
 	__tablename__ = 'signup'
 	id = Column(Integer(), primary_key=True, autoincrement=True)
 	token = Column(String(128), default=token_urlfriendly, nullable=False)
@@ -32,9 +34,10 @@ class Signup(db.Model):
 	loginname = Column(Text)
 	displayname = Column(Text)
 	mail = Column(Text)
-	pwhash = Column(Text)
-	user_dn = Column(String(128)) # Set after successful confirmation
-	user = DBRelationship('user_dn', User, backref='signups')
+	_password = Column('pwhash', Text)
+	password = PasswordHashAttribute('_password', LowEntropyPasswordHash)
+	user_id = Column(Integer(), ForeignKey('user.id', onupdate='CASCADE', ondelete='CASCADE'), nullable=True, unique=True)
+	user = relationship('User', backref=backref('signups', cascade='all, delete-orphan'))
 
 	type = Column(String(50))
 	__mapper_args__ = {
@@ -42,23 +45,26 @@ class Signup(db.Model):
 		'polymorphic_on': type
 	}
 
-	# Write-only property
-	def password(self, value):
+	def set_password(self, value):
 		if not User().set_password(value):
-			return
-		self.pwhash = crypt(value)
-	password = property(fset=password)
+			return False
+		self.password = value
+		return True
 
-	def check_password(self, value):
-		return self.pwhash is not None and crypt(value, self.pwhash) == self.pwhash
-
-	@property
+	@hybrid_property
 	def expired(self):
-		return self.created is not None and datetime.datetime.now() >= self.created + datetime.timedelta(hours=48)
+		if self.created is None:
+			return False
+		return self.created < datetime.datetime.now() - datetime.timedelta(hours=48)
 
-	@property
+	@hybrid_property
 	def completed(self):
-		return self.user_dn is not None
+		# pylint: disable=singleton-comparison
+		return self.user_id != None
+
+	@hybrid_property
+	def expired_and_not_completed(self):
+		return db.and_(self.expired, db.not_(self.completed))
 
 	def validate(self): # pylint: disable=too-many-return-statements
 		'''Return whether the signup request is valid and Signup.finish is likely to succeed
@@ -74,7 +80,7 @@ class Signup(db.Model):
 			return False, 'Display name is invalid'
 		if not User().set_mail(self.mail):
 			return False, 'Mail address is invalid'
-		if self.pwhash is None:
+		if not self.password:
 			return False, 'Invalid password'
 		if User.query.filter_by(loginname=self.loginname).all():
 			return False, 'A user with this login name already exists'
@@ -93,16 +99,16 @@ class Signup(db.Model):
 		          User object.'''
 		if self.completed or self.expired:
 			return None, 'Invalid signup request'
-		if not self.check_password(password):
+		if not self.password.verify(password):
 			return None, 'Wrong password'
 		if User.query.filter_by(loginname=self.loginname).all():
 			return None, 'A user with this login name already exists'
-		user = User(loginname=self.loginname, displayname=self.displayname, mail=self.mail, password=password)
-		ldap.session.add(user)
-		user.update_groups()
+		user = User(loginname=self.loginname, displayname=self.displayname, mail=self.mail, password=self.password)
+		db.session.add(user)
+		user.update_groups() # pylint: disable=no-member
 		self.user = user
 		self.loginname = None
 		self.displayname = None
 		self.mail = None
-		self.pwhash = None
+		self.password = None
 		return user, 'Success'
