@@ -1,12 +1,10 @@
 import functools
 import secrets
-import datetime
 
 from flask import Blueprint, render_template, request, redirect, url_for, flash, current_app, jsonify
 from flask_babel import gettext as _
 
 from uffd.database import db
-from uffd.ldap import ldap
 from uffd.session import set_session
 from uffd.user.models import User
 from uffd.sendmail import sendmail
@@ -59,7 +57,11 @@ def signup_submit():
 	host_ratelimit.log()
 	signup = Signup(loginname=request.form['loginname'],
 	                displayname=request.form['displayname'],
-	                mail=request.form['mail'], password=request.form['password1'])
+	                mail=request.form['mail'])
+	# If the password is invalid, signup.set_password returns False and does not
+	# set signup.password. We don't need to check the return value here, because
+	# we call signup.verify next and that checks if signup.password is set.
+	signup.set_password(request.form['password1'])
 	valid, msg = signup.validate()
 	if not valid:
 		return render_template('signup/start.html', error=msg)
@@ -71,19 +73,6 @@ def signup_submit():
 	signup_ratelimit.log(request.form['mail'])
 	return render_template('signup/submitted.html', signup=signup)
 
-# Deprecated
-@bp.route('/confirm/<token>')
-def signup_confirm_legacy(token):
-	matching_signup = None
-	filter_expr = Signup.created >= (datetime.datetime.now() - datetime.timedelta(hours=48))
-	for signup in Signup.query.filter(filter_expr):
-		if secrets.compare_digest(signup.token, token):
-			matching_signup = signup
-	if not matching_signup:
-		flash(_('Invalid signup link'))
-		return redirect(url_for('session.login'))
-	return redirect(url_for('signup.signup_confirm', signup_id=matching_signup.id, token=token))
-
 # signup_confirm* views are always accessible so other modules (e.g. invite) can reuse them
 @bp.route('/confirm/<int:signup_id>/<token>')
 def signup_confirm(signup_id, token):
@@ -105,7 +94,7 @@ def signup_confirm_submit(signup_id, token):
 		return render_template('signup/confirm.html', signup=signup, error=_('Too many failed attempts! Please wait %(delay)s.', delay=format_delay(confirm_delay)))
 	if host_delay:
 		return render_template('signup/confirm.html', signup=signup, error=_('Too many requests! Please wait %(delay)s.', delay=format_delay(host_delay)))
-	if not signup.check_password(request.form['password']):
+	if not signup.password.verify(request.form['password']):
 		host_ratelimit.log()
 		confirm_ratelimit.log(token)
 		return render_template('signup/confirm.html', signup=signup, error=_('Wrong password'))
@@ -113,7 +102,6 @@ def signup_confirm_submit(signup_id, token):
 	if user is None:
 		return render_template('signup/confirm.html', signup=signup, error=msg)
 	db.session.commit()
-	ldap.session.commit()
-	set_session(user, password=request.form['password'], skip_mfa=True)
+	set_session(user, skip_mfa=True)
 	flash(_('Your account was successfully created'))
 	return redirect(url_for('selfservice.index'))
